@@ -11,8 +11,8 @@
 #'
 #' @section Usage notes:
 #'
-#'     Call \code{\link{nano_init}} after package load to set global options.
-#'     Using defaults will cause warnings to print immediately as they occur.
+#'     Call \code{\link{nano_init}} after package load to set global options
+#'     such as causing warnings to print immediately as they occur.
 #'
 #'     \{nanonext\} offers 2 equivalent interfaces: an object-oriented interface,
 #'     and a functional interface.
@@ -126,14 +126,6 @@
 #'
 NULL
 
-.onLoad <- function(libname, pkgname) {
-  .logging. <- switch(tolower(Sys.getenv("NANONEXT_LOG")),
-                              info = TRUE,
-                              FALSE)
-  .logging. <<- .logging.
-  invisible()
-}
-
 .onUnload <- function(libpath) {
   if (!is.null(warn <- getOption("nanonext.original.warn"))) {
     options(warn = warn)
@@ -142,4 +134,3 @@ NULL
   invisible()
 }
 
-.logging. <- FALSE
