@@ -1,10 +1,10 @@
 # nanonext - Options Configuration ---------------------------------------------
 
-#' Set Option on Socket, Context, Dialer or Listener
+#' Set Option on Socket, Context, Dialer, Listener or Stream
 #'
-#' Set \link{opts} on a Socket, Context, Dialer or Listener.
+#' Set \link{opts} on a Socket, Context, Dialer, Listener or Stream.
 #'
-#' @param object a Socket, Context, Listener or Dialer.
+#' @param object a Socket, Context, Listener, Dialer or Stream.
 #' @param type [default 'bool'] type of option - either 'bool', 'int', 'ms'
 #'     (duration), 'size', 'string' or 'uint64'.
 #' @param opt name of option, e.g. 'reconnect-time-min', as a character string.
@@ -29,7 +29,6 @@ setopt <- function(object,
                    opt,
                    value) UseMethod("setopt")
 
-#'
 #' @examples
 #' s <- socket("pair")
 #' setopt(s, "ms", "recv-timeout", 2000)
@@ -44,81 +43,11 @@ setopt.nanoSocket <- function(object,
                               opt,
                               value) {
 
-  type <- match.arg(type)
-  xc <- switch(type,
-               bool = .Call(rnng_socket_set_bool, object, opt, value),
-               int = .Call(rnng_socket_set_int, object, opt, value),
-               ms = .Call(rnng_socket_set_ms, object, opt, value),
-               size = .Call(rnng_socket_set_size, object, opt, value),
-               string = .Call(rnng_socket_set_string, object, opt, value),
-               uint64 = .Call(rnng_socket_set_uint64, object, opt, value))
-
-  if (xc) logerror(xc)
-  invisible(xc)
+  type <- match.arg2(type, c("bool", "int", "ms", "size", "string", "uint64"))
+  invisible(.Call(rnng_socket_set, object, type, opt, value))
 
 }
 
-#'
-#' @examples
-#' s <- socket("pair", dial = "inproc://nanonext", autostart = FALSE)
-#' setopt(s$dialer[[1]], "ms", "reconnect-time-min", 2000)
-#' start(s$dialer[[1]])
-#' close(s)
-#'
-#' @rdname setopt
-#' @method setopt nanoDialer
-#' @export
-#'
-setopt.nanoDialer <- function(object,
-                              type = c("bool", "int", "ms", "size", "string", "uint64"),
-                              opt,
-                              value) {
-
-  type <- match.arg(type)
-  xc <- switch(type,
-               bool = .Call(rnng_dialer_set_bool, object, opt, value),
-               int = .Call(rnng_dialer_set_int, object, opt, value),
-               ms = .Call(rnng_dialer_set_ms, object, opt, value),
-               size = .Call(rnng_dialer_set_size, object, opt, value),
-               string = .Call(rnng_dialer_set_string, object, opt, value),
-               uint64 = .Call(rnng_dialer_set_uint64, object, opt, value))
-
-  if (xc) logerror(xc)
-  invisible(xc)
-
-}
-
-#'
-#' @examples
-#' s <- socket("pair", listen = "inproc://nanonext", autostart = FALSE)
-#' setopt(s$listener[[1]], "size", "recv-size-max", 1024)
-#' start(s$listener[[1]])
-#' close(s)
-#'
-#' @rdname setopt
-#' @method setopt nanoListener
-#' @export
-#'
-setopt.nanoListener <- function(object,
-                                type = c("bool", "int", "ms", "size", "string", "uint64"),
-                                opt,
-                                value) {
-
-  type <- match.arg(type)
-  xc <- switch(type,
-               bool = .Call(rnng_listener_set_bool, object, opt, value),
-               int = .Call(rnng_listener_set_int, object, opt, value),
-               ms = .Call(rnng_listener_set_ms, object, opt, value),
-               size = .Call(rnng_listener_set_size, object, opt, value),
-               string = .Call(rnng_listener_set_string, object, opt, value),
-               uint64 = .Call(rnng_listener_set_uint64, object, opt, value))
-
-  if (xc) logerror(xc)
-  invisible(xc)
-
-}
-
-#'
 #' @examples
 #' s <- socket("req")
 #' ctx <- context(s)
@@ -135,17 +64,62 @@ setopt.nanoContext <- function(object,
                                opt,
                                value) {
 
-  type <- match.arg(type)
-  xc <- switch(type,
-               bool = .Call(rnng_ctx_set_bool, object, opt, value),
-               int = .Call(rnng_ctx_set_int, object, opt, value),
-               ms = .Call(rnng_ctx_set_ms, object, opt, value),
-               size = .Call(rnng_ctx_set_size, object, opt, value),
-               string = .Call(rnng_ctx_set_string, object, opt, value),
-               uint64 = .Call(rnng_ctx_set_uint64, object, opt, value))
-
-  if (xc) logerror(xc)
-  invisible(xc)
+  type <- match.arg2(type, c("bool", "int", "ms", "size", "string", "uint64"))
+  invisible(.Call(rnng_ctx_set, object, type, opt, value))
+
+}
+
+#' @examples
+#' s <- socket("pair", dial = "inproc://nanonext", autostart = FALSE)
+#' setopt(s$dialer[[1]], "ms", "reconnect-time-min", 2000)
+#' start(s$dialer[[1]])
+#' close(s)
+#'
+#' @rdname setopt
+#' @method setopt nanoDialer
+#' @export
+#'
+setopt.nanoDialer <- function(object,
+                              type = c("bool", "int", "ms", "size", "string", "uint64"),
+                              opt,
+                              value) {
+
+  type <- match.arg2(type, c("bool", "int", "ms", "size", "string", "uint64"))
+  invisible(.Call(rnng_dialer_set, object, type, opt, value))
+
+}
+
+#' @examples
+#' s <- socket("pair", listen = "inproc://nanonext", autostart = FALSE)
+#' setopt(s$listener[[1]], "size", "recv-size-max", 1024)
+#' start(s$listener[[1]])
+#' close(s)
+#'
+#' @rdname setopt
+#' @method setopt nanoListener
+#' @export
+#'
+setopt.nanoListener <- function(object,
+                                type = c("bool", "int", "ms", "size", "string", "uint64"),
+                                opt,
+                                value) {
+
+  type <- match.arg2(type, c("bool", "int", "ms", "size", "string", "uint64"))
+  invisible(.Call(rnng_listener_set, object, type, opt, value))
+
+}
+
+#' @rdname setopt
+#' @method setopt nanoStream
+#' @export
+#'
+setopt.nanoStream <- function(object,
+                              type = c("bool", "int", "ms", "size", "string", "uint64"),
+                              opt,
+                              value) {
+
+  type <- match.arg2(type, c("bool", "int", "ms", "size", "string", "uint64"))
+  invisible(.Call(rnng_stream_set, object, type, opt, value))
 
 }
 
