@@ -160,7 +160,7 @@ SEXP rnng_aio_result(SEXP aio) {
 
 }
 
-SEXP rnng_aio_get_msg(SEXP aio) {
+SEXP rnng_aio_get_msg(SEXP aio, SEXP mode, SEXP keep) {
 
   if (R_ExternalPtrTag(aio) != nano_AioSymbol)
     error_return("object is not a valid Aio");
@@ -180,17 +180,15 @@ SEXP rnng_aio_get_msg(SEXP aio) {
   if (res)
     return mk_error(res);
 
+  const int mod = INTEGER(mode)[0], kpr = LOGICAL(keep)[0];
+  unsigned char *buf = nng_msg_body(raio->data);
   size_t sz = nng_msg_len(raio->data);
-  SEXP vec = PROTECT(Rf_allocVector(RAWSXP, sz));
-  unsigned char *rp = RAW(vec);
-  memcpy(rp, nng_msg_body(raio->data), sz);
 
-  UNPROTECT(1);
-  return vec;
+  return nano_decode(buf, sz, mod, kpr);
 
 }
 
-SEXP rnng_aio_stream_in(SEXP aio) {
+SEXP rnng_aio_stream_in(SEXP aio, SEXP mode, SEXP keep) {
 
   if (R_ExternalPtrTag(aio) != nano_AioSymbol)
     error_return("object is not a valid Aio");
@@ -210,14 +208,12 @@ SEXP rnng_aio_stream_in(SEXP aio) {
   if (res)
     return mk_error(res);
 
+  const int mod = INTEGER(mode)[0], kpr = LOGICAL(keep)[0];
   nng_iov *iov = (nng_iov *) iaio->data;
+  unsigned char *buf = iov->iov_buf;
   size_t sz = nng_aio_count(iaio->aio);
-  SEXP vec = PROTECT(Rf_allocVector(RAWSXP, sz));
-  unsigned char *rp = RAW(vec);
-  memcpy(rp, iov->iov_buf, sz);
 
-  UNPROTECT(1);
-  return vec;
+  return nano_decode(buf, sz, mod, kpr);
 
 }
 
@@ -252,11 +248,11 @@ SEXP rnng_aio_stop(SEXP aio) {
 
   nano_aio *aiop = (nano_aio *) R_ExternalPtrAddr(coreaio);
   nng_aio_stop(aiop->aio);
+  nng_aio_free(aiop->aio);
+  nng_mtx_free(aiop->mtx);
 
   nng_iov *iov;
   nano_handle *handle;
-  nng_aio_free(aiop->aio);
-  nng_mtx_free(aiop->mtx);
   switch (aiop->type) {
   case RECVAIO:
     if (aiop->data != NULL)
@@ -292,7 +288,8 @@ SEXP rnng_aio_stop(SEXP aio) {
 
 SEXP rnng_aio_unresolv(void) {
 
-  SEXP res = PROTECT(Rf_ScalarLogical(NA_LOGICAL));
+  SEXP res;
+  PROTECT(res = Rf_ScalarLogical(NA_LOGICAL));
   Rf_classgets(res, Rf_mkString("unresolvedValue"));
   UNPROTECT(1);
   return res;
@@ -318,11 +315,11 @@ SEXP rnng_recv_aio(SEXP socket, SEXP timeout) {
     error_return("'con' is not a valid Socket");
 
   nng_socket *sock = (nng_socket *) R_ExternalPtrAddr(socket);
-  nano_aio *raio;
-  int xc;
   const nng_duration dur = (nng_duration) Rf_asInteger(timeout);
+  nano_aio *raio = R_Calloc(1, nano_aio);
+  int xc;
+  SEXP aio;
 
-  raio = R_Calloc(1, nano_aio);
   raio->type = RECVAIO;
   raio->data = NULL;
 
@@ -341,7 +338,7 @@ SEXP rnng_recv_aio(SEXP socket, SEXP timeout) {
   nng_aio_set_timeout(raio->aio, dur);
   nng_recv_aio(*sock, raio->aio);
 
-  SEXP aio = PROTECT(R_MakeExternalPtr(raio, nano_AioSymbol, R_NilValue));
+  PROTECT(aio = R_MakeExternalPtr(raio, nano_AioSymbol, R_NilValue));
   R_RegisterCFinalizerEx(aio, raio_finalizer, TRUE);
 
   UNPROTECT(1);
@@ -355,11 +352,11 @@ SEXP rnng_ctx_recv_aio(SEXP context, SEXP timeout) {
     error_return("'con' is not a valid Context");
 
   nng_ctx *ctxp = (nng_ctx *) R_ExternalPtrAddr(context);
-  nano_aio *raio;
-  int xc;
   const nng_duration dur = (nng_duration) Rf_asInteger(timeout);
+  nano_aio *raio = R_Calloc(1, nano_aio);
+  int xc;
+  SEXP aio;
 
-  raio = R_Calloc(1, nano_aio);
   raio->type = RECVAIO;
   raio->data = NULL;
 
@@ -378,7 +375,7 @@ SEXP rnng_ctx_recv_aio(SEXP context, SEXP timeout) {
   nng_aio_set_timeout(raio->aio, dur);
   nng_ctx_recv(*ctxp, raio->aio);
 
-  SEXP aio = PROTECT(R_MakeExternalPtr(raio, nano_AioSymbol, R_NilValue));
+  PROTECT(aio = R_MakeExternalPtr(raio, nano_AioSymbol, R_NilValue));
   R_RegisterCFinalizerEx(aio, raio_finalizer, TRUE);
 
   UNPROTECT(1);
@@ -393,13 +390,13 @@ SEXP rnng_stream_recv_aio(SEXP stream, SEXP bytes, SEXP timeout) {
 
   nng_stream *sp = (nng_stream *) R_ExternalPtrAddr(stream);
   const nng_duration dur = (nng_duration) Rf_asInteger(timeout);
-  const size_t xlen = Rf_asInteger(bytes);
-  int xc;
-
+  const size_t xlen = (size_t) Rf_asInteger(bytes);
   nano_aio *iaio = R_Calloc(1, nano_aio);
-  iaio->type = 2;
-
   nng_iov *iov = R_Calloc(1, nng_iov);
+  int xc;
+  SEXP aio;
+
+  iaio->type = IOV_RECVAIO;
   iaio->data = iov;
   iov->iov_len = xlen;
   iov->iov_buf = R_Calloc(xlen, unsigned char);
@@ -419,7 +416,7 @@ SEXP rnng_stream_recv_aio(SEXP stream, SEXP bytes, SEXP timeout) {
     R_Free(iaio);
     return mk_error(xc);
   }
-  xc = nng_aio_set_iov(iaio->aio, 1, iaio->data);
+  xc = nng_aio_set_iov(iaio->aio, 1u, iov);
   if (xc) {
     nng_mtx_free(iaio->mtx);
     nng_aio_free(iaio->aio);
@@ -432,7 +429,7 @@ SEXP rnng_stream_recv_aio(SEXP stream, SEXP bytes, SEXP timeout) {
   nng_aio_set_timeout(iaio->aio, dur);
   nng_stream_recv(sp, iaio->aio);
 
-  SEXP aio = PROTECT(R_MakeExternalPtr(iaio, nano_AioSymbol, R_NilValue));
+  PROTECT(aio = R_MakeExternalPtr(iaio, nano_AioSymbol, R_NilValue));
   R_RegisterCFinalizerEx(aio, iraio_finalizer, TRUE);
 
   UNPROTECT(1);
@@ -448,15 +445,14 @@ SEXP rnng_send_aio(SEXP socket, SEXP data, SEXP timeout) {
     error_return("'con' is not a valid Socket");
 
   nng_socket *sock = (nng_socket *) R_ExternalPtrAddr(socket);
-  nano_aio *saio;
+  const nng_duration dur = (nng_duration) Rf_asInteger(timeout);
+  nano_aio *saio = R_Calloc(1, nano_aio);
   nng_msg *msg;
   int xc;
+  SEXP aio, enc = nano_encode(data);
+  const R_xlen_t xlen = Rf_xlength(enc);
+  unsigned char *dp = RAW(enc);
 
-  const nng_duration dur = (nng_duration) Rf_asInteger(timeout);
-  unsigned char *dp = RAW(data);
-  const R_xlen_t xlen = Rf_xlength(data);
-
-  saio = R_Calloc(1, nano_aio);
   saio->type = SENDAIO;
 
   xc = nng_msg_alloc(&msg, 0);
@@ -488,7 +484,7 @@ SEXP rnng_send_aio(SEXP socket, SEXP data, SEXP timeout) {
   nng_aio_set_timeout(saio->aio, dur);
   nng_send_aio(*sock, saio->aio);
 
-  SEXP aio = PROTECT(R_MakeExternalPtr(saio, nano_AioSymbol, R_NilValue));
+  PROTECT(aio = R_MakeExternalPtr(saio, nano_AioSymbol, R_NilValue));
   R_RegisterCFinalizerEx(aio, saio_finalizer, TRUE);
 
   UNPROTECT(1);
@@ -502,15 +498,14 @@ SEXP rnng_ctx_send_aio(SEXP context, SEXP data, SEXP timeout) {
     error_return("'con' is not a valid Context");
 
   nng_ctx *ctxp = (nng_ctx *) R_ExternalPtrAddr(context);
-  nano_aio *saio;
+  const nng_duration dur = (nng_duration) Rf_asInteger(timeout);
+  nano_aio *saio = R_Calloc(1, nano_aio);
   nng_msg *msg;
   int xc;
+  SEXP aio, enc = nano_encode(data);
+  const R_xlen_t xlen = Rf_xlength(enc);
+  unsigned char *dp = RAW(enc);
 
-  const nng_duration dur = (nng_duration) Rf_asInteger(timeout);
-  unsigned char *dp = RAW(data);
-  const R_xlen_t xlen = Rf_xlength(data);
-
-  saio = R_Calloc(1, nano_aio);
   saio->type = SENDAIO;
 
   xc = nng_msg_alloc(&msg, 0);
@@ -543,7 +538,7 @@ SEXP rnng_ctx_send_aio(SEXP context, SEXP data, SEXP timeout) {
   nng_aio_set_timeout(saio->aio, dur);
   nng_ctx_send(*ctxp, saio->aio);
 
-  SEXP aio = PROTECT(R_MakeExternalPtr(saio, nano_AioSymbol, R_NilValue));
+  PROTECT(aio = R_MakeExternalPtr(saio, nano_AioSymbol, R_NilValue));
   R_RegisterCFinalizerEx(aio, saio_finalizer, TRUE);
 
   UNPROTECT(1);
@@ -558,15 +553,15 @@ SEXP rnng_stream_send_aio(SEXP stream, SEXP data, SEXP timeout) {
 
   nng_stream *sp = (nng_stream *) R_ExternalPtrAddr(stream);
   const nng_duration dur = (nng_duration) Rf_asInteger(timeout);
-  unsigned char *dp = RAW(data);
-  const R_xlen_t xlen = Rf_xlength(data);
-  const int frames = LOGICAL(Rf_getAttrib(stream, nano_TextframesSymbol))[0];
-  int xc;
-
+  const int frames = *LOGICAL(Rf_getAttrib(stream, nano_TextframesSymbol));
   nano_aio *iaio = R_Calloc(1, nano_aio);
-  iaio->type = 1;
-
   nng_iov *iov = R_Calloc(1, nng_iov);
+  int xc;
+  SEXP aio, enc = nano_encode(data);
+  const R_xlen_t xlen = Rf_xlength(enc);
+  unsigned char *dp = RAW(enc);
+
+  iaio->type = IOV_SENDAIO;
   iaio->data = iov;
   iov->iov_len = frames == 1 ? xlen - 1 : xlen;
   iov->iov_buf = dp;
@@ -584,7 +579,7 @@ SEXP rnng_stream_send_aio(SEXP stream, SEXP data, SEXP timeout) {
     R_Free(iaio);
     return Rf_ScalarInteger(xc);
   }
-  xc = nng_aio_set_iov(iaio->aio, 1, iaio->data);
+  xc = nng_aio_set_iov(iaio->aio, 1u, iov);
   if (xc) {
     nng_mtx_free(iaio->mtx);
     nng_aio_free(iaio->aio);
@@ -596,7 +591,7 @@ SEXP rnng_stream_send_aio(SEXP stream, SEXP data, SEXP timeout) {
   nng_aio_set_timeout(iaio->aio, dur);
   nng_stream_send(sp, iaio->aio);
 
-  SEXP aio = PROTECT(R_MakeExternalPtr(iaio, nano_AioSymbol, R_NilValue));
+  PROTECT(aio = R_MakeExternalPtr(iaio, nano_AioSymbol, R_NilValue));
   R_RegisterCFinalizerEx(aio, isaio_finalizer, TRUE);
 
   UNPROTECT(1);
@@ -608,14 +603,16 @@ SEXP rnng_stream_send_aio(SEXP stream, SEXP data, SEXP timeout) {
 
 SEXP rnng_ncurl_aio(SEXP http, SEXP method, SEXP headers, SEXP data) {
 
-  int xc;
-  nano_aio *haio = R_Calloc(1, nano_aio);
-  haio->type = 3;
-  nano_handle *handle = R_Calloc(1, nano_handle);
-  handle->cfg = NULL;
-  haio->data = handle;
-
   const char *httr = CHAR(STRING_ELT(http, 0));
+  nano_aio *haio = R_Calloc(1, nano_aio);
+  nano_handle *handle = R_Calloc(1, nano_handle);
+  int xc;
+  SEXP aio;
+
+  haio->type = HTTP_AIO;
+  haio->data = handle;
+  handle->cfg = NULL;
+
   xc = nng_url_parse(&handle->url, httr);
   if (xc) {
     R_Free(handle);
@@ -651,7 +648,7 @@ SEXP rnng_ncurl_aio(SEXP http, SEXP method, SEXP headers, SEXP data) {
   }
   if (headers != R_NilValue) {
     R_xlen_t hlen = Rf_xlength(headers);
-    SEXP names = PROTECT(Rf_getAttrib(headers, R_NamesSymbol));
+    SEXP names = Rf_getAttrib(headers, R_NamesSymbol);
     switch (TYPEOF(headers)) {
     case STRSXP:
       for (R_xlen_t i = 0; i < hlen; i++) {
@@ -664,7 +661,6 @@ SEXP rnng_ncurl_aio(SEXP http, SEXP method, SEXP headers, SEXP data) {
           nng_url_free(handle->url);
           R_Free(handle);
           R_Free(haio);
-          UNPROTECT(1);
           return mk_error(xc);
         }
       }
@@ -680,17 +676,15 @@ SEXP rnng_ncurl_aio(SEXP http, SEXP method, SEXP headers, SEXP data) {
           nng_url_free(handle->url);
           R_Free(handle);
           R_Free(haio);
-          UNPROTECT(1);
           return mk_error(xc);
         }
       }
       break;
     }
-    UNPROTECT(1);
   }
   if (data != R_NilValue) {
     unsigned char *dp = RAW(data);
-    const R_xlen_t dlen = Rf_xlength(data) - 1;
+    const size_t dlen = Rf_xlength(data) - 1;
     xc = nng_http_req_set_data(handle->req, dp, dlen);
     if (xc) {
       nng_http_req_free(handle->req);
@@ -763,7 +757,7 @@ SEXP rnng_ncurl_aio(SEXP http, SEXP method, SEXP headers, SEXP data) {
 
   nng_http_client_transact(handle->cli, handle->req, handle->res, haio->aio);
 
-  SEXP aio = PROTECT(R_MakeExternalPtr(haio, nano_AioSymbol, R_NilValue));
+  PROTECT(aio = R_MakeExternalPtr(haio, nano_AioSymbol, R_NilValue));
   R_RegisterCFinalizerEx(aio, haio_finalizer, TRUE);
 
   UNPROTECT(1);
@@ -801,12 +795,12 @@ SEXP rnng_aio_http(SEXP aio) {
 
   void *dat;
   size_t sz;
+  SEXP vec;
+
   nng_http_res_get_data(handle->res, &dat, &sz);
-  SEXP vec = PROTECT(Rf_allocVector(RAWSXP, sz));
-  unsigned char *rp = RAW(vec);
-  memcpy(rp, dat, sz);
+  vec = Rf_allocVector(RAWSXP, sz);
+  memcpy(RAW(vec), dat, sz);
 
-  UNPROTECT(1);
   return vec;
 
 }
