@@ -34,6 +34,9 @@
 
 #ifdef NANONEXT_INTERNALS
 extern SEXP mk_error(const int);
+extern SEXP nano_decode(unsigned char *, size_t, const int, const int);
+extern SEXP nano_encode(SEXP);
+extern SEXP rawOneString(unsigned char *, R_xlen_t, R_xlen_t *);
 extern void socket_finalizer(SEXP);
 extern void dialer_finalizer(SEXP);
 extern void listener_finalizer(SEXP);
@@ -51,21 +54,22 @@ extern SEXP nano_SocketSymbol;
 extern SEXP nano_StateSymbol;
 extern SEXP nano_StreamSymbol;
 extern SEXP nano_TextframesSymbol;
+extern SEXP nano_UnserSymbol;
 extern SEXP nano_UrlSymbol;
 
 extern SEXP rnng_aio_call(SEXP);
-extern SEXP rnng_aio_get_msg(SEXP);
+extern SEXP rnng_aio_get_msg(SEXP, SEXP, SEXP);
 extern SEXP rnng_aio_http(SEXP);
 extern SEXP rnng_aio_result(SEXP);
 extern SEXP rnng_aio_stop(SEXP);
-extern SEXP rnng_aio_stream_in(SEXP);
+extern SEXP rnng_aio_stream_in(SEXP, SEXP, SEXP);
 extern SEXP rnng_aio_unresolv(void);
 extern SEXP rnng_close(SEXP);
 extern SEXP rnng_ctx_close(SEXP);
 extern SEXP rnng_ctx_open(SEXP);
-extern SEXP rnng_ctx_recv(SEXP, SEXP);
+extern SEXP rnng_ctx_recv(SEXP, SEXP, SEXP, SEXP);
 extern SEXP rnng_ctx_recv_aio(SEXP, SEXP);
-extern SEXP rnng_ctx_send(SEXP, SEXP, SEXP);
+extern SEXP rnng_ctx_send(SEXP, SEXP, SEXP, SEXP);
 extern SEXP rnng_ctx_send_aio(SEXP, SEXP, SEXP);
 extern SEXP rnng_ctx_set(SEXP, SEXP, SEXP, SEXP);
 extern SEXP rnng_dial(SEXP, SEXP);
@@ -78,22 +82,25 @@ extern SEXP rnng_listener_close(SEXP);
 extern SEXP rnng_listener_create(SEXP, SEXP);
 extern SEXP rnng_listener_set(SEXP, SEXP, SEXP, SEXP);
 extern SEXP rnng_listener_start(SEXP);
+extern SEXP rnng_matcharg(SEXP);
+extern SEXP rnng_matchargs(SEXP);
 extern SEXP rnng_messenger(SEXP);
 extern SEXP rnng_ncurl(SEXP, SEXP, SEXP, SEXP);
 extern SEXP rnng_ncurl_aio(SEXP, SEXP, SEXP, SEXP);
 extern SEXP rnng_protocol_open(SEXP);
-extern SEXP rnng_recv(SEXP, SEXP);
+extern SEXP rnng_recv(SEXP, SEXP, SEXP, SEXP);
 extern SEXP rnng_recv_aio(SEXP, SEXP);
 extern SEXP rnng_scm(void);
-extern SEXP rnng_send(SEXP, SEXP, SEXP);
+extern SEXP rnng_serial(SEXP);
+extern SEXP rnng_send(SEXP, SEXP, SEXP, SEXP);
 extern SEXP rnng_send_aio(SEXP, SEXP, SEXP);
 extern SEXP rnng_socket_set(SEXP, SEXP, SEXP, SEXP);
 extern SEXP rnng_stream_close(SEXP);
 extern SEXP rnng_stream_dial(SEXP, SEXP);
 extern SEXP rnng_stream_listen(SEXP, SEXP);
-extern SEXP rnng_stream_recv(SEXP, SEXP, SEXP);
+extern SEXP rnng_stream_recv(SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP rnng_stream_recv_aio(SEXP, SEXP, SEXP);
-extern SEXP rnng_stream_send(SEXP, SEXP, SEXP);
+extern SEXP rnng_stream_send(SEXP, SEXP, SEXP, SEXP);
 extern SEXP rnng_stream_send_aio(SEXP, SEXP, SEXP);
 extern SEXP rnng_stream_set(SEXP, SEXP, SEXP, SEXP);
 extern SEXP rnng_strerror(SEXP);
